## Extending KD tool

You can define subcommands:

    kd module command sub1 sub2 --paramkey paramval --paramkey1 paramval1 --parambool

This command will match these pattern:

```coffeescript
module.exports = class Module
  command: (sub1, sub2)->
    {paramkey, paramkey1, parambool} = @options

    # paramkey is paramval
    # paramkey1 is paramval1
    # parambool is true
```

## Modules

Modules are in `modules` directory. Every module is a file exporting a class.

Also you can create your modules in `.kd/modules` directory.

This is an example with a name `mymodule.coffee`

```coffeescript
module.exports = class MyModule

  # This closes the errors of the command. Not recommended.
  silent: yes

  help: """
  Koding MyModule Controller
  """

  alias:
    hi: "hello"

  constructor: (@config)->

  hello: (name)->
    {with} = @options
    console.log "hello #{name} and #{with}"

  __command: (command, params)->
    # magic command
```

### Kodingfile.coffee

You also can use `kd` with `Kodingfile.coffee` file. If a directory has that file kd will run it.
The command shouldn't be a module name. Because kd will search for existing modules first. Kodingfile
is the latest one it looks.

While using Kodingfile, you should use only the command name:

```coffeescript
module.exports = class Kodingfile
  hello: (name)->
    console.log "Hello, #{name}"
```

will run with

    kd hello fka

This command will search for "hello" module first, won't find and will look for your Kodingfile.coffee.

### Modules Meta

The `help` is an help to show user. When user call `kd mymodule` that information will be shown.

`@config` variable is the `~/.kdconfig` file. It's a JSON file and you can set global variables using `config` module (write `kd config`).

If you write `__command` into your module class, your module will never give a error about command existance. It'll call that method.

You can use `alias` to make aliases.

The example above can be run calling:

    kd mymodule hello koding --with birds

or with the alias:

    kd mymodule hello koding --with birds

The output will be:

    hello koding and birds