fs          = require "fs"
os          = require "os"
path        = require "path"
utils       = require "./utils"
{unique}    = require "underscore"
{logger}    = utils
log         = logger()
{findFile, guard} = require '../modules/common/helpers'

{ConfigFile, KD_DIR, MODULE_ROOT, CURRENT_MODULE_ROOT, USER_MODULE_ROOT} = require '../lib/configFile'

module.exports = class Dispatcher

  getKeys: ()=>
    readKeyFile = (f)=>
      # read and return key, if not exists key file, regenerate
      fpath = path.join KD_DIR, f
      try
        s = fs.readFileSync(fpath).toString()
        return s
      catch Error
        if not fs.existsSync KD_DIR
          fs.mkdirSync KD_DIR
        key = utils.keygen 64
        fs.writeFileSync fpath, key
        return key
    @publicKey  = readKeyFile("koding.key.pub")
    @privateKey = readKeyFile("koding.key") 


  missingParameterHelp: ()->
    console.log """

        Usage:

          kd #{module} <command> [arguments]

        The commands of the #{module} module are:

    """
    # TODO: get command names from help
    commands = (command for command, method of @moduleClass.prototype when typeof method is "function" and not command.match /__/).sort()
    console.log "  " + (command for command in commands).join ", "
    console.log ""


  findModulePath: (module)->
    # These should show errors inside modules
    # Loading module from the module path.
    lookup = [
      path.join CURRENT_MODULE_ROOT, "#{module}.coffee"
      path.join MODULE_ROOT, "#{module}.coffee"
      path.join USER_MODULE_ROOT, "#{module}.coffee"

      path.join CURRENT_MODULE_ROOT, "#{module}.js"
      path.join MODULE_ROOT, "#{module}.js"
      path.join USER_MODULE_ROOT, "#{module}.js"

      path.join CURRENT_MODULE_ROOT, "#{module}.kdmodule", "index.coffee"
      path.join MODULE_ROOT, "#{module}.kdmodule", "index.coffee"
      path.join USER_MODULE_ROOT, "#{module}.kdmodule", "index.coffee"

      path.join CURRENT_MODULE_ROOT, "#{module}.kdmodule", "index.js"
      path.join MODULE_ROOT, "#{module}.kdmodule", "index.js"
      path.join USER_MODULE_ROOT, "#{module}.kdmodule", "index.js"
    ]

    for modulePath in lookup
      p = path.normalize modulePath
      if fs.existsSync p 
        return p


  constructor: (@module, @command, @params)->
    @root = process.cwd()
    @getKeys()

    if not @command
      @command = 'default'

    @configFile = new ConfigFile
    @configFile.config.publicKey = @publicKey
    @configFile.config.privateKey = @privateKey

    if not @module
      @module = 'help'

    # Replace module with the alias
    if @configFile.config.alias?[module]
      module = @configFile.config.alias[module]

    modulePath = @findModulePath @module
    guard not modulePath, "Module \"#{module}\" not found."

    @moduleClass = require modulePath

    {help, silent} = @moduleClass.prototype


    fnname = @moduleClass.commands[@command]
    if not fnname
        return @missingParameterHelp()

    try
      @moduleClass.utils   = require path.join __dirname, "utils.coffee"
      @moduleInstance      = new @moduleClass @configFile
    catch error
      console.log "ERROR: Module instance couldn't be created."
      console.trace error

    fn = @moduleInstance[fnname]
    guard not fn, "ERROR: command #{command} not found"

    @moduleInstance.options = require "optimist"
    @moduleInstance[fnname](@params...)

  @run: (coffeeBin, file, module, command, params...) =>
    new @ module, command, params
