fs      = require "fs"
os      = require "os"
colors  = require "colors"
{spawn} = require "child_process"

shuffle = (array) ->
  i = array.length
  return if i is 0

  while --i
    j = Math.floor Math.random() * ( i + 1 )
    temp = array[i]
    array[i] = array[j]
    array[j] = temp

random = (max)-> Math.floor Math.random() * max

keygen = (length, options = {}) ->
  keys = []
  chars = "0123456789abcdefghijklmnopqrstuvwxyz"
  rest = length
  for i in [0..length]
    rand = random chars.length
    upper = length * random(length) % 2
    char = chars.substring rand, rand+1
    unless options.lower
      char = if upper then char.toUpperCase() else char
    keys.push char
  keys.join ""
  shuffle(keys).join ""

ask = (question, {format, hidden, callback, error})->
  {stdin, stdout} = process
  stdout.write "#{question} "
  stdin.resume()
  stdin.setEncoding "utf8"
  unless hidden
    stdin.once "data", (answer)->
      answer = answer.toString().trim()
      if format?.test? answer
        callback answer
      else
        if error
          error answer.toString().trim()
        else
          stdout.write "Please enter a valid answer.\n"
        ask question, {format, callback, error}
  else
    stdin.setRawMode true
    stdout.write "\n"
    buffer = []
    stdin.on "data", (key)->
      process.exit 0 if key is "\u0003" or key is "\u0004"
      if key is "\u000D"
        answer = buffer.join ""
        callback answer
      else
        key = key.toString().trim()
        buffer.push key

logger = (namespace="kd")->(message, color="yellow")->
  message?.toString?().split("\n").forEach (line)->
    nocolor = ~process.argv.indexOf "--nocolor"
    if nocolor
      console.log "#{namespace}:", line
    else
      console.log colors[color]("#{namespace}:"), line

compileDebug = (path, source, error)->
  data = source.toString()
  if error.location
    {first_line, last_line, first_column, last_column} = error.location
    lines = data.split os.EOL
    trace = lines.slice(first_line, last_line+1).join os.EOL
    point = ""
    for i in [0..last_column]
      if i < first_column then point+=" " else point+="^"
    point+= " #{error.message}"

  first_line++; last_line++;
  spaces = Array((first_line+"").length+1).join " "
  curr = first_line
  prev = first_line - 1
  next = first_line + 1

  previous_line = if first_line > 1 then "#{prev}   #{lines[prev-1]}" else ""
  next_line = if lines.length > next then "#{next}  #{lines[next-1]}" else ""
  """

  at #{path} line #{first_line}:#{last_line} column #{first_column}:#{last_column}

  #{previous_line}
  #{colors.red curr}   #{colors.red trace}
  #{spaces}   #{colors.green point}
  #{next_line}
  """

spawner = ({exec, params, data, error, exit})->
  timeStart = Date.now()
  child = spawn executable, params
  child.stdout.on "data",(data)-> data? data
  child.stderr.on "data",(data)-> error? data
  child.on "exit",(code)-> 
    exit? code, "child pid:#{child.pid} was alive #{(Date.now()-timeStart)/1000}msecs"
  return child

module.exports = {shuffle, random, keygen, ask, logger, compileDebug, spawner}
