fs          = require "fs"
os          = require "os"
colors      = require "colors"
path        = require "path"
url         = require "url"
http        = require "http"
https       = require "https"
request     = require "request"
qs          = require "querystring"
utils       = require "../lib/utils"
Controller  = require '../lib/controller'

{findFile, guard} = require '../modules/common/helpers'


{ask, logger} = utils
log     = logger "register"

module.exports = class Register extends Controller
  KD_DIR = path.join process.env.HOME, ".kd"

  @set
    commands:
        'renew': ['renew', 'Renews your kite public key']
        'set': ['set', 'Sets your koding key']
        'register': ['register', 'Registers your computer']
        'default': 'register'

  constructor: (@configFile)->
    @privateKeyPath = path.join KD_DIR, "koding.key"
    @publicKeyPath = path.join KD_DIR, "koding.key.pub"
    @configFile.config.KODING_ROOT or= "https://koding.com"

  get: ->
    console.log @configFile.config.publicKey

  renew: ->
    {argv: {all, silent}} = @options
    change = (path, name)=>
      key = utils.keygen 64
      fs.writeFileSync path, key
      console.log "#{name}: #{key}"

    unless silent
      ask "Do you really want to do it? It may affect your works! (yes|no)", 
        format: /yes|no|Y|N/,
        callback: (answer)=>
          if answer is "yes"
            change @publicKeyPath, "Public"
            if all then change @privateKeyPath, "Private"
          process.exit()
        error: (answer)->
          console.log "Please write a valid answer: yes or no. That's simple."
    else
      change @publicKeyPath, "Public"
      if all then change @privateKeyPath, "Private"

  where: ->
    console.log @publicKeyPath

  set: (key)->
    unless key?.length is 64
      log "Your key's length must be 64.", "red"
      process.exit 1
    fs.writeFileSync (path.join process.env.HOME, ".kd", "koding.key.pub"), key
    log "You've changed your key successfully.", "green"

  register: =>
    guard not @configFile.config.publicKey, "you need to set your public key"

    Config = require '../modules/config'
    if not @configFile.config['user.name']
      # HELLO first timer
      readpath = path.join( __dirname, '..', 'modules/resources/kd-manifest.json' )
      rs = fs.createReadStream(readpath)
      rs.pipe(fs.createWriteStream(path.join(KD_DIR, "kd-manifest.json" )))

      c = new Config(@configFile)
      c.username (configuration)=>
        @configFile = configuration
        @_register()
    else
      @_register()

  _register: ->
    guard not @configFile.config["user.name"], "you need to set your username"
    guard not @configFile.config.publicKey, "you need to set your public key"

    {argv: {verbose}} = @options
      .alias("v", "verbose")
      .alias("t", "timeout")

    link = "#{@configFile.config.KODING_ROOT}/-/KD/Register/#{encodeURIComponent os.hostname()}/#{@configFile.config.publicKey}"
    log """


    Hello #{@configFile.config["user.name"]}!

    Please visit the following URL to connect your 
    host to this computer.


    """

    console.log "#{colors.underline link}"
    
    trycount = 0
    process.stdout.write "register: "
    process.stdout.write "Waiting for you to connect the host"
    waiting = setInterval =>
      check?.abort?()

      client = require('needle')

      data = 
        key: @configFile.config.publicKey
        username : @configFile.config["user.name"]


      url = "#{@configFile.config.KODING_ROOT}/-/kd/register-check"

      client.post url, data, multipart:true, (err, resp, body)->
        console.log(err)
        console.log("Got status code " + resp.statusCode)
        console.log(body)

        if resp.statusCode isnt 200
          console.log "An error occured with the status #{resp.statusCode}: #{http.STATUS_CODES[resp.statusCode]}."
          return

        if body is "OK"
          console.log "Host connected!"
          clearInterval waiting
        else
          console.log "Registration not found. Retrying."
          console.log "response was:", body
        trycount += 1

        if trycount > 10
          clearInterval waiting
          console.log "registration timed out please re-run"

    , 5000
