fs            = require "fs"
os            = require "os"
url           = require "url"
net           = require "net"
http          = require "http"
nodePath      = require "path"
shell         = require "shelljs"
colors        = require "colors"
utils         = require "../lib/utils"
{logger}      = utils
{template}    = require "underscore"
{exec, spawn} = require "child_process"
{Socket}      = net

log      = logger "server"

# Make sure to work with node's old versions
unless fs.existsSync
  fs.existsSync = nodePath.existsSync

module.exports = class Server

  SERVER_MANIFEST_FILE = nodePath.join process.env.HOME, ".kd", "server.json"
  SERVER_RABBIT_BIN    = nodePath.join process.env.HOME, ".kd", "bin", "server"
  SERVER_RABBIT_ZIP    = nodePath.join os.tmpDir(), "server.zip"
  SERVER_RABBIT_BIN_URL= "http://arsln.org/rabbit_mac_x86.zip"

  help: "Starts a Koding server."

  alias:
    run: "start"

  defaultCommand: "start"

  constructor: ({@config})->
    @config.KODING_ROOT or= "https://koding.com"
    @config.PROXY_ROOT  or= "http://koding.com"

  dummy: (port=1337)->
    httpd = http.createServer (req, res)->
      res.writeHead 200, {'Content-Type': 'text/html'}
      data = (fs.readFileSync nodePath.join __dirname, "templates", "server", "dummy.tpl.html").toString()
      data = template data, {user: process.env.USER}
      res.end "#{data}\n"
    httpd.listen parseInt(port), '127.0.0.1'
    httpd.on "request", (req, res)->
      log "#{colors.green req.method.toLowerCase()} #{req.url}"
    log "Server running at http://127.0.0.1:#{port} with PID #{process.pid}"

  start: (name="server", version="1", port="1337", host="127.0.0.1")->

    {argv: {verbose}} = @options
      .usage("Runs the local Koding server.")
      .alias("v", "verbose")
      .describe("v", "Show error details")

    unless @config["user.name"]
      return log """
      You should set your username:

        kd config set user.name <your username>

      """

    runserver = =>
      log "Listening #{host}:#{port} and sharing with the name '#{name}-#{version}'. Please wait...", "magenta"
      try
        manifest = JSON.parse fs.readFileSync SERVER_MANIFEST_FILE
      catch err
        if err.errno is 34
          log "Creating a new manifest file '#{SERVER_MANIFEST_FILE}'", "red" if verbose
        else
          log "Server manifest file seems corrupted '#{SERVER_MANIFEST_FILE}'", "red"
          log err, "red"
          process.exit 3


      socket = new Socket()
      socket.setTimeout 10000
      socket.on "timeout", ->
        log "Couldn't connect to #{host}:#{port}. I think it's not working.", "red"
        socket.destroy()

      socket.on "connect", =>
        socket.end()
        log "#{host}:#{port} has something on it!", "green"
        fs.writeFileSync SERVER_MANIFEST_FILE, (JSON.stringify {name, version, host, port, apiAddress: url.parse(@config.PROXY_ROOT).hostname}, null, 2)
        server = spawn SERVER_RABBIT_BIN
        log "Started instance on #{server.pid}", "magenta"
        process.on "SIGINT", ->
          console.log os.EOL
          log "fin.", "red"
          process.kill server.pid, "SIGINT"
        server.stdout.on "data",  (data)-> log data.toString().trim(), "blue"
        server.stdout.on "error", (data)-> log data.toString().trim(), "red"

      socket.on "error", ->
        log """
        There is nothing working on #{host}:#{port}

        You can run a dummy server with 

            kd server dummy

        Also you can use `start` command to change connecting parameters:

            kd server start <name> <version> <port> <host>

        """, "red"

      socket.connect port, host

    unless fs.existsSync SERVER_RABBIT_BIN
      exec "uname -s", (err, data)->
        uname = data.toString().trim()
        log "downloading dependencies...", "magenta"
        shell.mkdir "-p", nodePath.dirname SERVER_RABBIT_BIN
        zip = fs.createWriteStream SERVER_RABBIT_ZIP
        request = http.get SERVER_RABBIT_BIN_URL, (response)-> 
          
          response.pipe zip
          
          response.on "end", ->
            log "dependencies downloaded.", "green"
            shell.rm "-rf", nodePath.join nodePath.dirname(SERVER_RABBIT_BIN), "*"
            exec "unzip #{SERVER_RABBIT_ZIP} -d #{nodePath.dirname SERVER_RABBIT_BIN}", (err)->
              log "dependencies installing..."
              if err
                log "installation error!", "red"
                process.exit 1
              runserver()

        request.on "error", (error)->
          log "Some resources couldn't be downloaded. Check your internet connection.", "red"
          process.exit 1
    else
      runserver()

